<?php

/**
 * @file
 * Contains \Drupal\linkicon\Plugin\Field\FieldFormatter\LinkIconFormatter.
 */

namespace Drupal\linkicon\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\String;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Url;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'linkicon' formatter.
 *
 * @FieldFormatter(
 *   id = "linkicon",
 *   label = @Translation("Link icon, based on title"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkIconFormatter extends LinkFormatter {

  /**
   * The linkicon service.
   *
   * @var \Drupal\linkicon\LinkIconManagerInterface.
   */
  protected $linkIconManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('linkicon.manager')
    );
  }

  /**
   * Constructs a new LinkIconFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\linkicon\LinkIconManagerInterface $linkicon_manager
   *   The linkicon service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LinkIconManagerInterface $linkicon_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->linkIconManager = $linkicon_manager;
  }

  /**
   * Returns the linkicon plugin manager.
   *
   * @return \Drupal\linkicon\LinkIconManagerInterface
   *   The linkicon plugin manager.
   */
  protected function linkIconManager() {
    if (!$this->linkIconManager) {
      $this->linkIconManager = \Drupal::service('linkicon.manager');
    }

    return $this->linkIconManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'rel' => '',
      'target' => '',
      'linkicon_prefix' => 'icon',
      'linkicon_wrapper_class' => '',
      'linkicon_load' => FALSE,
      'linkicon_vertical' => FALSE,
      'linkicon_style' => '',
      'linkicon_color' => '',
      'linkicon_tooltip' => FALSE,
      'linkicon_maxlength' => '',
      'linkicon_no_text' => FALSE,
      'linkicon_position' => '',
      'linkicon_global_title' => '',
      'linkicon_size' => '',
      'linkicon_bundle' => '',
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    $path     = drupal_get_path('module', 'linkicon');
    $settings = $this->getSettings();

    // Predefined titles are supposed to be controlled.
    if (isset($elements['trim_length'])) {
      unset($elements['trim_length']);
    }

    $elements['opening'] = array(
      '#markup' => '<h3>' . $this->t('If your theme has no icon font library, define one <a href="@url" target="_blank">here</a>.', array('@url' => \Drupal::url('linkicon.settings'))) . '</h3>',
    );
    $elements['linkicon_prefix'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Icon prefix class'),
      '#description' => $this->t('A "prefix" or "namespace", e.g.: icon (Fontello), fa (FontAwesome), st-icon (Stackicons), genericon, fonticon, etc. <br />If the link title is <em>Facebook</em>, it will create classes: <em>icon icon-facebook</em> for Fontello, or <em>fa fa-facebook</em> for FontAwesome > 3. <br />The individual icon class itself is based on the link text key matching the pattern: icon-KEY, or fa-KEY.'),
      '#default_value' => $settings['linkicon_prefix'],
      '#required' => TRUE,
    );

    $elements['linkicon_wrapper_class'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Additional wrapper class'),
      '#description' => $this->t('Additional wrapper class for the entire icon list apart from <strong>item-list item-list--linkicon</strong>.'),
      '#default_value' => $settings['linkicon_wrapper_class'],
    );

    $elements['linkicon_load'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Allow linkicon to provide CSS assets'),
      '#default_value' => $settings['linkicon_load'],
      '#description' => $this->t('Otherwise, DIY accordingly.'),
    );

    $elements['linkicon_vertical'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Vertical'),
      '#default_value' => $settings['linkicon_vertical'],
      '#description' => $this->t('By default, icons are displayed inline. Check to make icons stacked vertically.'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_style'] = array(
      '#type' => 'select',
      '#title' => $this->t('Icon style'),
      '#default_value' => $settings['linkicon_style'],
      '#options' => array(
        'round'    => t('Round'),
        'round-2'  => t('Round 2'),
        'round-5'  => t('Round 5'),
        'round-8'  => t('Round 8'),
        'round-10' => t('Round 10'),
        'square'   => t('Square'),
        'button'   => t('Button'),
      ),
      '#empty_option' => $this-> t('- None -'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
      '#description' => $this->t('Button is more prominent if the title text is not hidden over the background color.'),
    );

    $elements['linkicon_color'] = array(
      '#type' => 'select',
      '#title' => $this->t('Icon background color'),
      '#default_value' => $settings['linkicon_color'],
      '#options' => array(
        'grey'   => t('Grey'),
        'dark'   => t('Dark'),
        'purple' => t('Purple'),
        'orange' => t('Orange'),
        'blue'   => t('Blue'),
        'lime'   => t('Lime'),
        'red'    => t('Red'),
      ),
      '#empty_option' => $this-> t('- None -'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
      '#description' => $this->t('Basic background color. You should do proper theming to suit your design better, and disable all this.'),
    );

    $elements['linkicon_tooltip'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Display title as tooltip'),
      '#default_value' => $settings['linkicon_tooltip'],
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_no_text'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Visually hide the title text'),
      '#default_value' => $settings['linkicon_no_text'],
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    $elements['linkicon_maxlength'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('The title and tooltip maxlength'),
      '#description' => $this->t('Limit the amount of characters if using token replacement, default to 60 characters.'),
      '#default_value' => $settings['linkicon_maxlength'],
      '#size' => 6,
      '#maxlength' => 3,
    );

    $elements['linkicon_global_title'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Override title with a generic link title'),
      '#description' => $this->t('If provided, the link title will be overriden with this text, e.g.: Visit the site, View Demo. Token is supported.'),
      '#default_value' => $settings['linkicon_global_title'],
      '#states' => array(
        'visible' => array(
          array(':input[name*="linkicon_tooltip"]' => array('checked' => FALSE)),
          array(':input[name*="linkicon_no_text"]' => array('checked' => FALSE)),
        ),
      ),
    );

    $elements['linkicon_position'] = array(
      '#type' => 'select',
      '#title' => $this->t('Icon position to the title text.'),
      '#default_value' => $settings['linkicon_position'],
      '#description' => $this->t('By default icon is before the text - Left.'),
      '#options' => array(
        'bottom' => t('Bottom'),
        'right'  => t('Right'),
        'top'    => t('Top'),
      ),
      '#empty_option' => $this-> t('Left'),
      '#states' => array(
        'visible' => array(
          array(':input[name*="linkicon_load"]' => array('checked' => TRUE)),
          array(':input[name*="linkicon_no_text"]' => array('checked' => FALSE)),
        ),
      ),
    );

    $icon_sizes = array(
      'small' => t('Small'),
      'medium' => t('Medium'),
      'large' => t('Large'),
      'xlarge' => t('X-large'),
      'xxlarge' => t('Xx-large'),
    );

    $elements['linkicon_size'] = array(
      '#type' => 'select',
      '#title' => $this->t('Icon font size'),
      '#default_value' => $settings['linkicon_size'],
      '#options' => $icon_sizes,
      '#empty_option' => $this-> t('Default'),
      '#states' => array(
        'visible' => array(
          ':input[name*="linkicon_load"]' => array('checked' => TRUE),
        ),
      ),
    );

    // Build a preview.
    if (function_exists('icon_bundles')) {
      $bundles   = icon_bundles();
      $providers = icon_providers();

      $provider_options = array();
      foreach ($providers as $name => $provider) {
        $provider_options[$provider['name']] = $provider['title'];
      }

      if ($provider_options) {
        $elements['linkicon_bundle'] = array(
          '#type' => 'select',
          '#title' => $this->t('Icon module integration'),
          '#options' => $provider_options,
          '#default_value' => $settings['linkicon_bundle'],
          '#description' => $this->t('The above icon providers modules are detected. You can choose which icon CSS file to load with this module. <br>Make sure that you have a working module that loads relevant CSS accordingly. <br>Known working modules as of this writing: fontawesome, and icomoon.'),
          '#empty_option' => $this-> t('- None -'),
        );
      }
    }

    $has_icon_path = FALSE;
    if ($this->linkIconManager->getSetting('font')) {
      $has_icon_path = TRUE;
    }

    if (!empty($settings['linkicon_bundle']) || $has_icon_path) {
      $icon_previews = array();
      $linkicon_item = array(
        '#theme' => 'linkicon_item',
        '#position' => $settings['linkicon_position'],
        '#title' => 'Twitter',
        '#attributes' => array('class' => array('linkicon__icon', $settings['linkicon_prefix'], $settings['linkicon_prefix'] . '-twitter')),
        '#icon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : FALSE,
        '#icon_name'   => 'twitter',
      );

      $icon = drupal_render($linkicon_item);

      $tooltip = '';
      if ($settings['linkicon_tooltip']) {
        $tooltip = ' data-title="Twitter"';
      }

      foreach ($icon_sizes as $key => $size) {
        $is_active = $key == $settings['linkicon_size'] ? ' active' : '';
        $icon_previews[] = array('#markup' => '<a class="linkicon__item linkicon--' . $key . $is_active . '" href="#"' . $tooltip . '>' . $icon . '</a>');
      }

      $preview = array(
        '#theme' => 'linkicon',
        '#linkicon_id' => 'linkicon-preview',
        '#items' => $icon_previews,
        '#config' => array(
          'style' => $settings['linkicon_style'],
          'color' => $settings['linkicon_color'],
          'no_text' => $settings['linkicon_no_text'],
          'position' => $settings['linkicon_position'],
          'tooltip' => $settings['linkicon_tooltip'],
          'wrapper_class' => $settings['linkicon_wrapper_class'],
          'load' => $settings['linkicon_load'],
        ),
      );

      if ($settings['linkicon_load']) {
        if ($has_icon_path) {
          $elements['#attached']['library'][] = 'linkicon/linkicon.font';
        }
        $elements['#attached']['library'][] = 'linkicon/linkicon';
      }

      $elements['linkicon_size_preview'] = array(
        '#markup' => drupal_render($preview),
      );
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $settings = $this->getSettings();

    if (!empty($settings['rel'])) {
      $summary[] = t('Add rel="@rel"', array('@rel' => $settings['rel']));
    }
    if (!empty($settings['target'])) {
      $summary[] = t('Open link in new window');
    }

    $summary[] = t('Base class: <em>@linkicon_prefix</em>.', array(
    '@linkicon_prefix' => $settings['linkicon_prefix'],
    ));

    $summary[] = t('Module CSS: <em>@linkicon_load</em>. Wrapper: <em>@linkicon_wrapper_class</em>. Style: <em>@linkicon_style</em>. Bg: <em>@linkicon_color</em>.', array(
      '@linkicon_load' => $settings['linkicon_load'] ? t('Yes') : t('No'),
      '@linkicon_wrapper_class' => $settings['linkicon_wrapper_class'] ? $settings['linkicon_wrapper_class'] : t('None'),
      '@linkicon_vertical' => $settings['linkicon_vertical'] ? t('Vertical') : t('Horizontal'),
      '@linkicon_style' => $settings['linkicon_style'] ? $settings['linkicon_style'] : t('None'),
      '@linkicon_color' => $settings['linkicon_color'] ? $settings['linkicon_color'] : t('None'),
    ));

    if ($settings['linkicon_load']) {
      $summary[] = t('Size: <em>@linkicon_size</em>. No text: <em>@linkicon_no_text</em>. Tooltip: <em>@linkicon_tooltip</em>', array(
        '@linkicon_size' => $settings['linkicon_size'],
        '@linkicon_no_text' => $settings['linkicon_no_text'] ? t('Yes') : t('No'),
        '@linkicon_tooltip' => $settings['linkicon_tooltip'] ? t('Yes') : t('No'),
      ));

      if (empty($settings['linkicon_no_text'])) {
        $summary[] = t('Use global title: <em>@linkicon_global_title</em>. <br>Icon position: <em>@linkicon_position</em>.', array(
          '@linkicon_global_title' => $settings['linkicon_global_title'] ? $settings['linkicon_global_title'] : t('No'),
          '@linkicon_position' => $settings['linkicon_position'] ? $settings['linkicon_position'] : t('Left'),
        ));
      }

      // @todo icon API integration, none by now 3/3/14
      // $summary[] = t('Icon provider: <em>@linkicon_bundle</em>.', array(
      // '@linkicon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : t('No'),
      // ));
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareView(array $entities_items) {
    // @todo drop 'predefined' for the new integer: LINKICON_PREDEFINED.
    if ($this->getFieldSetting('title') == LINKICON_PREDEFINED || $this->getFieldSetting('title') == 'predefined') {
      $settings = $this->getFieldSettings();
      if (!empty($settings['title_predefined'])) {
        $title_predefined = $this->linkIconManager->extractAllowedValues($settings['title_predefined']);
        $title_tooltip    = $this->linkIconManager->extractAllowedValues($settings['title_predefined'], TRUE);

        foreach ($entities_items as $entity_id => $items) {
          $new_values = array();
          foreach ($items as $delta => $item) {
            $values = $item->getValue();
            $new_values['display_title'] = $title_predefined[$values['title']];
            if (isset($title_tooltip[$values['title']])) {
              $new_values['tooltip'] = $title_tooltip[$values['title']];
            }
            $merged_values = array_merge($values, $new_values);
            $item->setValue($merged_values);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items) {
    $path         = drupal_get_path('module', 'linkicon');
    $element      = array();
    $contents     = array();
    $entity       = $items->getEntity();
    $entity_type  = $entity->getEntityTypeId();
    $bundle       = $entity->bundle();
    $id           = $entity->id();
    $settings     = $this->getSettings();
    $field_name   = $this->fieldDefinition->getName();

    foreach ($items as $delta => $item) {
      $attributes = array('class' => array('linkicon__item'));

      // Linkicon requires both link text and URL available with proper
      // validation during input, no need extra checks.
      $icon_name     = $item->title;
      $display_title = isset($item->display_title) ? $item->display_title : $icon_name;
      $tooltip       = isset($item->tooltip) ? $item->tooltip : $display_title;

      // @todo recheck if any icon class has any silly Uppercase.
      $icon_class = Html::cleanCssIdentifier(Unicode::strtolower($settings['linkicon_prefix'] . '-' . $icon_name));

      // If title is overriden with a generic one, make sure the $icon_class
      // is not overridden.
      if (!empty($settings['linkicon_global_title']) && empty($settings['linkicon_no_text'])) {
        $display_title = $settings['linkicon_global_title'];
      }

      $url = $this->buildUrl($item);
      $options = $url->getOptions();
      $options['html'] = TRUE;
      // Tokenized text is sanitized by default.
      $token = \Drupal::token();
      $display_title = $token->replace($display_title, array($entity_type => $entity));
      $tooltip = $token->replace($tooltip, array($entity_type => $entity));
      $character_maxlength = $settings['linkicon_maxlength'] ?: 60;
      $icon_element = array(
        '#theme'       => 'linkicon_item',
        '#position'    => $settings['linkicon_position'],
        '#title'       => Unicode::truncate($display_title, $character_maxlength, TRUE, TRUE),
        '#icon_bundle' => $settings['linkicon_bundle'] ? $settings['linkicon_bundle'] : FALSE,
        '#icon_name'   => String::checkPlain($icon_name),
        '#attributes'  => array(
          'class' => array(
            'linkicon__icon',
            $settings['linkicon_prefix'],
            $icon_class
          ),
          'aria-hidden' => 'true',
        ),
      );

      // Our pure CSS3 tooltip depends on data-title.
      if ($settings['linkicon_tooltip']) {
        $attributes['data-title'] = Unicode::truncate($tooltip, $character_maxlength, TRUE, TRUE);
      }

      // Merge with core options: rel and target.
      if ($options['attributes']) {
        $options['attributes'] += $attributes;
      }
      else {
        $options['attributes'] = $attributes;
      }

      // We are done, pass it over to link to do its job.
      // @todo https://www.drupal.org/node/2350519
      $contents[$delta] = array(
        '#type' => 'link',
        '#title' => $icon_element,
        '#url' => $url,
        '#options' => $options,
      );
    }

    // Build own wrapper for greater control.
    $linkicon_id = Html::cleanCssIdentifier("{$entity_type}-{$bundle}-{$field_name}-{$id}");
    $element = array(
      '#theme' => 'linkicon',
      '#linkicon_id' => 'linkicon-' . $linkicon_id,
      '#items' => $contents,
      '#config' => array(
        'style'         => $settings['linkicon_style'],
        'color'         => $settings['linkicon_color'],
        'size'          => $settings['linkicon_size'],
        'no_text'       => $settings['linkicon_no_text'],
        'position'      => $settings['linkicon_position'],
        'tooltip'       => $settings['linkicon_tooltip'],
        'vertical'      => $settings['linkicon_vertical'],
        'wrapper_class' => $settings['linkicon_wrapper_class'],
        'load'          => $settings['linkicon_load'],
      ),
      '#attributes' => array(
        'id' => 'linkicon-' . $linkicon_id,
      ),
    );

    // Attached our assets if so configured.
    if ($this->linkIconManager->getSetting('font')) {
      $element['#attached']['library'][] = 'linkicon/linkicon.font';
    }
    if ($settings['linkicon_load']) {
      $element['#attached']['library'][] = 'linkicon/linkicon';
    }

    return $element;
  }

}
